/* ssh-keyget.c
**
** Copyright (c) 1997 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
   
#include <stdio.h>
#include <netdb.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


int main(int argc, char *argv[])
{
    struct sockaddr_in sin;
    int fd, key_fd, len, nlen, i;
    char buf[32768], path[2048];
    char *key_dir = "/etc";

    if (argc < 2 || argc > 3)
    {
	fprintf(stderr, "Usage: ssh-keyget <host> <port> [<directory>]\n");
	exit(1);
    }
    
    if (argc == 3)
	key_dir = argv[3];

    fd = socket(AF_INET, SOCK_STREAM, 0);

    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    for (i = 1023; i > 512; i--)
    {
	sin.sin_port = htons(i);
	if (bind(fd, (struct sockaddr *) &sin, sizeof(sin)) == 0)
	    break;
    }
    if (i <= 512)
    {
	perror("keyget: warning: bind() to low port failed");
    }
    
    sin.sin_family = AF_INET;
    if (isdigit(argv[1][0]))
	sin.sin_addr.s_addr = inet_addr(argv[1]);
    else
    {
	struct hostent *hp;

	hp = gethostbyname(argv[1]);
	if (hp == NULL)
	{
	    fprintf(stderr, "keyget: unknown host: %s\n", argv[1]);
	    exit(1);
	}

	sin.sin_addr.s_addr = * (long *) hp->h_addr_list[0];
    }

    if (argc > 1)
	sin.sin_port = htons(atoi(argv[2]));
    else
	sin.sin_port = htons(1022);

    if (connect(fd, (struct sockaddr *) &sin, sizeof(sin)) < 0)
    {
	perror("keyget: connect");
	exit(1);
    }

    /* Get the private key */
    if (read(fd, &nlen, sizeof(nlen)) < sizeof(nlen))
    {
	fprintf(stderr, "keyget: short read(private key length)\n");
	exit(1);
    }
    
    len = ntohl(nlen);
    if (len > sizeof(buf))
    {
	fprintf(stderr, "keyget: key too long (%d > %d)\n",
		len, sizeof(buf));
	exit(1);
    }

    sprintf(buf, "%s/ssh_host_key", key_dir);
    key_fd = open(buf, O_CREAT+O_WRONLY+O_TRUNC, 0600);
    if (key_fd < 0)
    {
	perror("open(ssh_host_key)");
	exit(1);
    }

    if (read(fd, buf, len) < len)
    {
	fprintf(stderr, "keyget: short read(private key data)\n");
	exit(1);
    }

    write(key_fd, buf, len);
    close(key_fd);


    /* Get the public key */
    if (read(fd, &nlen, sizeof(nlen)) < sizeof(nlen))
    {
	fprintf(stderr, "keyget: short read(public key length)\n");
	exit(1);
    }
	
    len = ntohl(nlen);
    if (len > sizeof(buf))
    {
	fprintf(stderr, "keyget: key too long (%d > %d)\n",
		len, sizeof(buf));
	exit(1);
    }

    sprintf(buf, "%s/ssh_host_key.pub", key_dir);
    key_fd = open(buf, O_CREAT+O_WRONLY+O_TRUNC, 0644);
    if (key_fd < 0)
    {
	perror("open(ssh_host_key.pub)");
	exit(1);
    }

    if (read(fd, buf, len) < len)
    {
	fprintf(stderr, "keyget: short read(public key data)\n");
	exit(1);
    }

    write(key_fd, buf, len);
    close(key_fd);

    exit(0);
}
